@@ -4,9 +4,9 @@
 //! or you could specify a configuration file. The format of configuration file is defined
 //! in mod `config`.
 
-use std::{process, time::Duration};
+use std::{net::IpAddr, path::PathBuf, process, time::Duration};
 
-use clap::{clap_app, Arg};
+use clap::{clap_app, Arg, Error as ClapError, ErrorKind as ClapErrorKind};
 use futures::future::{self, Either};
 use log::info;
 use tokio::{self, runtime::Builder};
@@ -18,7 +18,8 @@ use shadowsocks_service::shadowsocks::relay::socks5::Address;
 use shadowsocks_service::{
     acl::AccessControl,
     config::{Config, ConfigType, LocalConfig, ProtocolType},
-    run_local,
+    create_local,
+    local::loadbalancing::PingBalancer,
     shadowsocks::{
         config::{Mode, ServerAddr, ServerConfig},
         crypto::v1::{available_ciphers, CipherKind},
@@ -41,12 +42,12 @@ fn main() {
             (version: VERSION)
             (about: "A fast tunnel proxy that helps you bypass firewalls.")
 
-            (@arg CONFIG: -c --config +takes_value required_unless_all(&["LOCAL_ADDR", "SERVER_CONFIG"]) "Shadowsocks configuration file (https://shadowsocks.org/en/config/quick-guide.html)")
+            (@arg CONFIG: -c --config +takes_value required_unless("SERVER_CONFIG") "Shadowsocks configuration file (https://shadowsocks.org/en/config/quick-guide.html)")
 
             (@arg LOCAL_ADDR: -b --("local-addr") +takes_value {validator::validate_server_addr} "Local address, listen only to this address if specified")
             (@arg UDP_ONLY: -u conflicts_with[TCP_AND_UDP] requires[LOCAL_ADDR] "Server mode UDP_ONLY")
             (@arg TCP_AND_UDP: -U requires[LOCAL_ADDR] "Server mode TCP_AND_UDP")
-            (@arg PROTOCOL: --protocol +takes_value requires[LOCAL_ADDR] possible_values(ProtocolType::available_protocols()) "Protocol for communicating with clients (SOCKS5 by default)")
+            (@arg PROTOCOL: --protocol +takes_value possible_values(ProtocolType::available_protocols()) "Protocol for communicating with clients (SOCKS5 by default)")
             (@arg UDP_BIND_ADDR: --("udp-bind-addr") +takes_value requires[LOCAL_ADDR] {validator::validate_server_addr} "UDP relay's bind address, default is the same as local-addr")
 
             (@arg SERVER_ADDR: -s --("server-addr") +takes_value {validator::validate_server_addr} requires[PASSWORD ENCRYPT_METHOD] "Server address")
@@ -76,6 +77,9 @@ fn main() {
             (@arg INBOUND_RECV_BUFFER_SIZE: --("inbound-recv-buffer-size") +takes_value {validator::validate_u32} "Set inbound sockets' SO_RCVBUF option")
             (@arg OUTBOUND_SEND_BUFFER_SIZE: --("outbound-send-buffer-size") +takes_value {validator::validate_u32} "Set outbound sockets' SO_SNDBUF option")
             (@arg OUTBOUND_RECV_BUFFER_SIZE: --("outbound-recv-buffer-size") +takes_value {validator::validate_u32} "Set outbound sockets' SO_RCVBUF option")
+
+            (@arg OUTBOUND_BIND_ADDR: --("outbound-bind-addr") +takes_value alias("bind-addr") {validator::validate_ip_addr} "Bind address, outbound socket will bind this address")
+            (@arg OUTBOUND_BIND_INTERFACE: --("outbound-bind-interface") +takes_value "Set SO_BINDTODEVICE / IP_BOUND_IF / IP_UNICAST_IF option for outbound socket")
         );
 
         // FIXME: -6 is not a identifier, so we cannot build it with clap_app!
@@ -111,18 +115,10 @@ fn main() {
         #[cfg(any(target_os = "linux", target_os = "android"))]
         {
             app = clap_app!(@app (app)
-                (@arg OUTBOUND_BIND_INTERFACE: --("outbound-bind-interface") +takes_value "Set SO_BINDTODEVICE option for outbound socket")
                 (@arg OUTBOUND_FWMARK: --("outbound-fwmark") +takes_value {validator::validate_u32} "Set SO_MARK option for outbound socket")
             );
         }
 
-        #[cfg(any(target_os = "macos", target_os = "ios"))]
-        {
-            app = clap_app!(@app (app)
-                (@arg OUTBOUND_BIND_INTERFACE: --("outbound-bind-interface") +takes_value "Set IP_BOUND_IF option for outbound socket")
-            );
-        }
-
         #[cfg(feature = "local-redir")]
         {
             if RedirType::tcp_default() != RedirType::NotSupported {
@@ -168,6 +164,21 @@ fn main() {
             }
         }
 
+        #[cfg(feature = "local-tun")]
+        {
+            app = clap_app!(@app (app)
+                (@arg TUN_INTERFACE_NAME: --("tun-interface-name") +takes_value "Tun interface name, allocate one if not specify")
+                (@arg TUN_INTERFACE_ADDRESS: --("tun-interface-address") +takes_value {validator::validate_ipnet} "Tun interface address (network)")
+            );
+
+            #[cfg(unix)]
+            {
+                app = clap_app!(@app (app)
+                    (@arg TUN_DEVICE_FD_FROM_PATH: --("tun-device-fd-from-path") +takes_value "Tun device file descriptor will be transferred from this unix domain socket path")
+                );
+            }
+        }
+
         #[cfg(unix)]
         {
             app = clap_app!(@app (app)
@@ -258,9 +269,7 @@ fn main() {
             config.outbound_vpn_protect_path = Some(From::from("protect_path"));
         }
 
-        if let Some(local_addr) = matches.value_of("LOCAL_ADDR") {
-            let local_addr = local_addr.parse::<ServerAddr>().expect("local bind addr");
-
+        if matches.value_of("LOCAL_ADDR").is_some() || matches.value_of("PROTOCOL").is_some() {
             let protocol = match matches.value_of("PROTOCOL") {
                 Some("socks") => ProtocolType::Socks,
                 #[cfg(feature = "local-http")]
@@ -271,11 +280,31 @@ fn main() {
                 Some("redir") => ProtocolType::Redir,
                 #[cfg(feature = "local-dns")]
                 Some("dns") => ProtocolType::Dns,
+                #[cfg(feature = "local-tun")]
+                Some("tun") => ProtocolType::Tun,
                 Some(p) => panic!("not supported `protocol` \"{}\"", p),
                 None => ProtocolType::Socks,
             };
 
-            let mut local_config = LocalConfig::new(local_addr, protocol);
+            let mut local_config = LocalConfig::new(protocol);
+            match (protocol, matches.value_of("LOCAL_ADDR")) {
+                #[cfg(feature = "local-tun")]
+                (ProtocolType::Tun, local_addr_opt) => {
+                    if let Some(local_addr) = local_addr_opt {
+                        local_config.addr = Some(local_addr.parse::<ServerAddr>().expect("local bind addr"));
+                    }
+                }
+                (_, None) => {
+                    ClapError::with_description(
+                        format!("Protocol \"{}\" requires local-addr", protocol.as_str()).as_str(),
+                        ClapErrorKind::ArgumentNotFound,
+                    )
+                    .exit();
+                }
+                (_, Some(local_addr)) => {
+                    local_config.addr = Some(local_addr.parse::<ServerAddr>().expect("local bind addr"));
+                }
+            }
 
             if let Some(udp_bind_addr) = matches.value_of("UDP_BIND_ADDR") {
                 local_config.udp_addr = Some(udp_bind_addr.parse::<ServerAddr>().expect("udp-bind-addr"));
@@ -321,7 +350,7 @@ fn main() {
                 if let Some(dns_relay_addr) = matches.value_of("DNS_LOCAL_ADDR") {
                     let addr = dns_relay_addr.parse::<ServerAddr>().expect("dns relay address");
 
-                    let mut local_dns_config = LocalConfig::new(addr, ProtocolType::Dns);
+                    let mut local_dns_config = LocalConfig::new_with_addr(addr, ProtocolType::Dns);
 
                     // The `local_dns_addr` and `remote_dns_addr` are for this DNS server (for compatibility)
                     local_dns_config.local_dns_addr = local_config.local_dns_addr.take();
@@ -331,6 +360,21 @@ fn main() {
                 }
             }
 
+            #[cfg(feature = "local-tun")]
+            {
+                if let Some(tun_address) = matches.value_of("TUN_INTERFACE_ADDRESS") {
+                    local_config.tun_interface_address = Some(tun_address.parse().expect("tun-interface-address"));
+                }
+                if let Some(tun_name) = matches.value_of("TUN_INTERFACE_NAME") {
+                    local_config.tun_interface_name = Some(tun_name.to_owned());
+                }
+
+                #[cfg(unix)]
+                if let Some(fd_path) = matches.value_of("TUN_DEVICE_FD_FROM_PATH") {
+                    local_config.tun_device_fd_from_path = Some(fd_path.into());
+                }
+            }
+
             if matches.is_present("UDP_ONLY") {
                 local_config.mode = Mode::UdpOnly;
             }
@@ -363,7 +407,6 @@ fn main() {
             config.outbound_fwmark = Some(mark.parse::<u32>().expect("an unsigned integer for `outbound-fwmark`"));
         }
 
-        #[cfg(any(target_os = "linux", target_os = "android", target_os = "macos", target_os = "ios"))]
         if let Some(iface) = matches.value_of("OUTBOUND_BIND_INTERFACE") {
             config.outbound_bind_interface = Some(iface.to_owned());
         }
@@ -413,6 +456,11 @@ fn main() {
             config.outbound_recv_buffer_size = Some(bs.parse::<u32>().expect("outbound-recv-buffer-size"));
         }
 
+        if let Some(bind_addr) = matches.value_of("OUTBOUND_BIND_ADDR") {
+            let bind_addr = bind_addr.parse::<IpAddr>().expect("outbound-bind-addr");
+            config.outbound_bind_addr = Some(bind_addr);
+        }
+
         // DONE READING options
 
         if config.local.is_empty() {
@@ -442,12 +490,12 @@ fn main() {
         }
 
         #[cfg(unix)]
-        if matches.is_present("DAEMONIZE") {
+        if matches.is_present("DAEMONIZE") || matches.is_present("DAEMONIZE_PID_PATH") {
             use self::common::daemonize;
             daemonize::daemonize(matches.value_of("DAEMONIZE_PID_PATH"));
         }
 
-        info!("shadowsocks {}", VERSION);
+        info!("shadowsocks local {} build {}", VERSION, common::BUILD_TIME);
 
         #[cfg(feature = "multi-threaded")]
         let mut builder = if matches.is_present("SINGLE_THREADED") {
@@ -468,8 +516,16 @@ fn main() {
     };
 
     runtime.block_on(async move {
+        let config_path = config.config_path.clone();
+
+        let instance = create_local(config).await.expect("create local");
+
+        if let Some(config_path) = config_path {
+            launch_reload_server_task(config_path, instance.server_balancer().clone());
+        }
+
         let abort_signal = monitor::create_signal_monitor();
-        let server = run_local(config);
+        let server = instance.run();
 
         tokio::pin!(abort_signal);
         tokio::pin!(server);
@@ -477,8 +533,8 @@ fn main() {
         match future::select(server, abort_signal).await {
             // Server future resolved without an error. This should never happen.
             Either::Left((Ok(..), ..)) => {
-                eprintln!("server exited unexpectly");
-                process::exit(common::EXIT_CODE_SERVER_EXIT_UNEXPECTLY);
+                eprintln!("server exited unexpectedly");
+                process::exit(common::EXIT_CODE_SERVER_EXIT_UNEXPECTEDLY);
             }
             // Server future resolved with error, which are listener errors in most cases
             Either::Left((Err(err), ..)) => {
@@ -490,3 +546,33 @@ fn main() {
         }
     });
 }
+
+#[cfg(unix)]
+fn launch_reload_server_task(config_path: PathBuf, balancer: PingBalancer) {
+    use log::error;
+    use tokio::signal::unix::{signal, SignalKind};
+
+    tokio::spawn(async move {
+        let mut sigusr1 = signal(SignalKind::user_defined1()).expect("signal");
+
+        while sigusr1.recv().await.is_some() {
+            let config = match Config::load_from_file(&config_path, ConfigType::Local) {
+                Ok(c) => c,
+                Err(err) => {
+                    error!("auto-reload {} failed with error: {}", config_path.display(), err);
+                    continue;
+                }
+            };
+
+            let servers = config.server;
+            info!("auto-reload {} with {} servers", config_path.display(), servers.len());
+
+            if let Err(err) = balancer.reset_servers(servers).await {
+                error!("auto-reload {} but found error: {}", config_path.display(), err);
+            }
+        }
+    });
+}
+
+#[cfg(not(unix))]
+fn launch_reload_server_task(_: PathBuf, _: PingBalancer) {}
