@@ -4,7 +4,7 @@
 //! or you could specify a configuration file. The format of configuration file is defined
 //! in mod `config`.
 //!
-//! *It should be notice that the extented configuration file is not suitable for the server
+//! *It should be notice that the extended configuration file is not suitable for the server
 //! side.*
 
 use std::{net::IpAddr, process, time::Duration};
@@ -43,7 +43,8 @@ fn main() {
 
             (@arg CONFIG: -c --config +takes_value required_unless("SERVER_ADDR") "Shadowsocks configuration file (https://shadowsocks.org/en/config/quick-guide.html)")
 
-            (@arg BIND_ADDR: -b --("bind-addr") +takes_value {validator::validate_ip_addr} "Bind address, outbound socket will bind this address")
+            (@arg OUTBOUND_BIND_ADDR: -b --("outbound-bind-addr") +takes_value alias("bind-addr") {validator::validate_ip_addr} "Bind address, outbound socket will bind this address")
+            (@arg OUTBOUND_BIND_INTERFACE: --("outbound-bind-interface") +takes_value "Set SO_BINDTODEVICE / IP_BOUND_IF / IP_UNICAST_IF option for outbound socket")
 
             (@arg SERVER_ADDR: -s --("server-addr") +takes_value {validator::validate_server_addr} requires[PASSWORD ENCRYPT_METHOD] "Server address")
             (@arg PASSWORD: -k --password +takes_value requires[SERVER_ADDR] "Server's password")
@@ -86,7 +87,7 @@ fn main() {
         {
             app = clap_app!(@app (app)
                 (@arg DAEMONIZE: -d --("daemonize") "Daemonize")
-                (@arg DAEMONIZE_PID_PATH: --("daemonize-pid") +takes_value "File path to store daemonized process's PID")
+                (@arg DAEMONIZE_PID_PATH: -f --("daemonize-pid") +takes_value "File path to store daemonized process's PID")
             );
         }
 
@@ -100,18 +101,10 @@ fn main() {
         #[cfg(any(target_os = "linux", target_os = "android"))]
         {
             app = clap_app!(@app (app)
-                (@arg OUTBOUND_BIND_INTERFACE: --("outbound-bind-interface") +takes_value "Set SO_BINDTODEVICE option for outbound socket")
                 (@arg OUTBOUND_FWMARK: --("outbound-fwmark") +takes_value {validator::validate_u32} "Set SO_MARK option for outbound socket")
             );
         }
 
-        #[cfg(any(target_os = "macos", target_os = "ios"))]
-        {
-            app = clap_app!(@app (app)
-                (@arg OUTBOUND_BIND_INTERFACE: --("outbound-bind-interface") +takes_value "Set IP_BOUND_IF option for outbound socket")
-            );
-        }
-
         #[cfg(feature = "multi-threaded")]
         {
             app = clap_app!(@app (app)
@@ -177,6 +170,9 @@ fn main() {
                 sc.set_plugin(plugin);
             }
 
+            // For historical reason, servers that are created from command-line have to be tcp_only.
+            sc.set_mode(Mode::TcpOnly);
+
             if matches.is_present("UDP_ONLY") {
                 sc.set_mode(Mode::UdpOnly);
             }
@@ -188,9 +184,9 @@ fn main() {
             config.server.push(sc);
         }
 
-        if let Some(bind_addr) = matches.value_of("BIND_ADDR") {
-            let bind_addr = bind_addr.parse::<IpAddr>().expect("bind-addr");
-            config.local_addr = Some(bind_addr);
+        if let Some(bind_addr) = matches.value_of("OUTBOUND_BIND_ADDR") {
+            let bind_addr = bind_addr.parse::<IpAddr>().expect("outbound-bind-addr");
+            config.outbound_bind_addr = Some(bind_addr);
         }
 
         if matches.is_present("TCP_NO_DELAY") {
@@ -214,7 +210,6 @@ fn main() {
             config.outbound_fwmark = Some(mark.parse::<u32>().expect("an unsigned integer for `outbound-fwmark`"));
         }
 
-        #[cfg(any(target_os = "linux", target_os = "android", target_os = "macos", target_os = "ios"))]
         if let Some(iface) = matches.value_of("OUTBOUND_BIND_INTERFACE") {
             config.outbound_bind_interface = Some(iface.to_owned());
         }
@@ -287,12 +282,12 @@ fn main() {
         }
 
         #[cfg(unix)]
-        if matches.is_present("DAEMONIZE") {
+        if matches.is_present("DAEMONIZE") || matches.is_present("DAEMONIZE_PID_PATH") {
             use self::common::daemonize;
             daemonize::daemonize(matches.value_of("DAEMONIZE_PID_PATH"));
         }
 
-        info!("shadowsocks {}", VERSION);
+        info!("shadowsocks server {} build {}", VERSION, common::BUILD_TIME);
 
         #[cfg(feature = "multi-threaded")]
         let mut builder = if matches.is_present("SINGLE_THREADED") {
@@ -322,8 +317,8 @@ fn main() {
         match future::select(server, abort_signal).await {
             // Server future resolved without an error. This should never happen.
             Either::Left((Ok(..), ..)) => {
-                eprintln!("server exited unexpectly");
-                process::exit(common::EXIT_CODE_SERVER_EXIT_UNEXPECTLY);
+                eprintln!("server exited unexpectedly");
+                process::exit(common::EXIT_CODE_SERVER_EXIT_UNEXPECTEDLY);
             }
             // Server future resolved with error, which are listener errors in most cases
             Either::Left((Err(err), ..)) => {
